package io.iohk.ethereum.jsonrpc

import akka.util.ByteString
import io.iohk.ethereum.domain.{BlockHeader, SignedTransaction}

final case class TransactionResponse(
    hash: ByteString,
    nonce: BigInt,
    blockHash: Option[ByteString],
    blockNumber: Option[BigInt],
    transactionIndex: Option[BigInt],
    from: Option[ByteString],
    to: Option[ByteString],
    value: BigInt,
    gasPrice: BigInt,
    gas: BigInt,
    input: ByteString,
    pending: Option[Boolean],
    isOutgoing: Option[Boolean])

final case class TransactionData(
  stx: SignedTransaction,
  blockHeader: Option[BlockHeader] = None,
  transactionIndex: Option[Int] = None,
  pending: Option[Boolean] = None,
  isOutgoing: Option[Boolean] = None
)

object TransactionResponse {

  def apply(tx: TransactionData): TransactionResponse =
    TransactionResponse(tx.stx, tx.blockHeader, tx.transactionIndex, tx.pending, tx.isOutgoing)

  def apply(stx: SignedTransaction,
            blockHeader: Option[BlockHeader] = None,
            transactionIndex: Option[Int] = None,
            pending: Option[Boolean] = None,
            isOutgoing: Option[Boolean] = None): TransactionResponse =
    TransactionResponse(
      hash = stx.hash,
      nonce = stx.tx.nonce,
      blockHash = blockHeader.map(_.hash),
      blockNumber = blockHeader.map(_.number),
      transactionIndex = transactionIndex.map(txIndex => BigInt(txIndex)),
      from = SignedTransaction.getSender(stx).map(_.bytes),
      to = stx.tx.receivingAddress.map(_.bytes),
      value = stx.tx.value,
      gasPrice = stx.tx.gasPrice,
      gas = stx.tx.gasLimit,
      input = stx.tx.payload,
      pending = pending,
      isOutgoing = isOutgoing)

}
